import React, {useState} from "react";
import {Form, Calendar, CapsuleTabs, JumboTabs, Picker, PickerView, Popup, CalendarPickerView} from "antd-mobile";
import {basicColumns} from "./constant";
import "./index.css"
import {CloseCircleFill} from "antd-mobile-icons";
import dayjs from "dayjs";

export default function DataPickItemPopup(props) {
    const {fieldName, labelName, disabled} = props
    const [visiblePopup, setVisiblePopup] = useState(false);
    const [dateShowValue, setDateShowValue] = useState('日期');
    const [timeValue, setTimeValue] = useState(['时间']);
    const [activeKey, setActiveKey] = useState("date");
    const [dataPick, setDataPick] = useState();
    return (
        <Form.Item noStyle
                   shouldUpdate={(prevValues, curValues) => {
                       return prevValues[fieldName] !== curValues[fieldName]
                   }}
        >
            {({getFieldValue, setFieldsValue}) => (
                <Form.Item
                    name={fieldName}
                    label={labelName}
                    disabled={disabled}
                    trigger='onConfirm'
                    arrow={
                        getFieldValue(fieldName) ? (
                            <CloseCircleFill
                                style={{
                                    color: 'var(--adm-color-light)',
                                    fontSize: 14,
                                }}
                                onClick={e => {
                                    e.stopPropagation()
                                    // 计算属性名:允许你在对象字面量中使用表达式来动态确定属性名。
                                    setFieldsValue({[fieldName]: null})
                                }}/>) : true
                    }
                    onClick={() => {
                        setVisiblePopup(true)
                        let fieldValue = getFieldValue(fieldName);
                        console.log({fieldValue})
                        if (fieldValue) {
                            let dayjs1 = dayjs(fieldValue);
                            setDateShowValue(dayjs1.format('YYYY-MM-DD'));
                            setTimeValue([dayjs1.hour().toString(), dayjs1.minute().toString()])
                            setDataPick(dayjs1.toDate())
                        } else {
                            let dayjs1 = dayjs(new Date());
                            setDateShowValue(dayjs1.format('YYYY-MM-DD'));
                            setTimeValue([dayjs1.hour().toString(), dayjs1.minute().toString()])
                            setDataPick(new Date())
                        }
                    }}
                >
                    {getFieldValue(fieldName) ? dayjs(getFieldValue(fieldName)).format('YYYY-MM-DD HH:mm') : '请选择日期'}
                    <Popup
                        visible={visiblePopup}
                        onMaskClick={() => {
                            setVisiblePopup(false)
                        }}
                        bodyStyle={{
                            borderTopLeftRadius: '8px',
                            borderTopRightRadius: '8px',
                            minHeight: '40vh',
                        }}
                    >
                        <CapsuleTabs className="tabs-container"
                                     onChange={(key) => {
                                         if (key === 'ok') {
                                             setVisiblePopup(false)
                                             if (dateShowValue && timeValue) {
                                                 setFieldsValue({
                                                     [fieldName]: dayjs(dateShowValue + ' ' + timeValue.join(":")).toDate()
                                                 })
                                             }
                                             setActiveKey('date')
                                         } else {
                                             setActiveKey(key)
                                         }
                                     }}
                                     defaultActiveKey={activeKey}
                        >
                            <CapsuleTabs.Tab title={dateShowValue}
                                             key='date' className="tab-content">
                                <CalendarPickerView
                                    selectionMode='single'
                                    defaultValue={dataPick}
                                    onChange={val => {
                                        if (val) {
                                            setDateShowValue(dayjs(val).format('YYYY-MM-DD'))
                                        } else {
                                            setDateShowValue(undefined)
                                        }
                                    }}
                                />

                            </CapsuleTabs.Tab>
                            <CapsuleTabs.Tab title={timeValue.join(":")} key='time'
                                             className="tab-content"
                            >
                                <PickerView
                                    columns={basicColumns}
                                    onClose={() => {
                                    }}
                                    defaultValue={timeValue}
                                    value={timeValue}
                                    onChange={(val, extend) => {
                                        console.log('onChange',timeValue, val, extend.items)
                                        setTimeValue(val)
                                    }}
                                />
                            </CapsuleTabs.Tab>
                            <CapsuleTabs.Tab title="确认" key="ok" destroyOnClose={true}>
                            </CapsuleTabs.Tab>
                        </CapsuleTabs>
                    </Popup>
                </Form.Item>
            )}
        </Form.Item>)

}