import {ActionSheet, Button, Dialog, Divider, Tabs, TextArea, Toast} from "antd-mobile";
import React, {Fragment, useEffect, useMemo, useRef, useState} from "react";
import "./index.css"
import {useNavigate, useOutletContext, useSearchParams} from "react-router-dom";
import {addTaskLog, deleteTaskLogById, listTaskLog, updateTaskLogEnableById} from "../../api/detailLogTaskApi";
import dayjs from "dayjs";
import {copyTextToClipboard} from "../../utils/copyToClipBoard";
import {getPTask} from "../../utils";
import InfiniteScroll from "react-infinite-scroll-component";

export function DetailLogTask() {
    // 设置标题栏
    const {setTitle, setRightDesc} = useOutletContext();
    useEffect(() => {
        setTitle("日记/心得")
        setRightDesc(<Fragment/>);
    }, [])
    let [params] = useSearchParams();
    // 路由
    const navigate = useNavigate();

    // 发送展示日记
    const [currentShow, setCurrentShow] = useState('need');
    const textAreaRef = useRef(null);

    const [taskLogList, setTaskLogList] = useState([]);
    const [sendValue, setSendValue] = useState('');

    const [hasMore, setHasMore] = useState(true);
    const [pageTime, setPageTime] = useState(new Date());

    // 记录上一次的滚动高度
    const previousScrollHeight = useRef(0);

    // 点击操作面板
    const [actionSheetVisible, setActionSheetVisible] = useState(false)
    const [currentTask, setCurrentTask] = useState({});
    const [actions, setActions] = useState([
        {text: '复制', key: 'copy'},
        {text: '有效', key: 'need'},
        {text: '创建任务', key: 'addTask'},
        {text: '删除', key: 'delete'},
    ]);

    const taskLogMapMemory = useMemo(() => {
        return taskLogList
            .filter(taskLog => {
                if (currentShow === 'all') {
                    return true
                } else if (currentShow === 'need' && taskLog.enableFlag === '1') {
                    return true
                } else return currentShow === 'noNeed' && taskLog.enableFlag === '0';
            })
            .reduce((map, taskLog) => {
                if (!map.has(dayjs(taskLog.createdDate).format("YYYY-MM-DD"))) {
                    map.set(dayjs(taskLog.createdDate).format("YYYY-MM-DD"), []);
                }
                map.get(dayjs(taskLog.createdDate).format("YYYY-MM-DD"))?.push(taskLog);
                return map;
            }, new Map());
    }, [taskLogList, currentShow])
    const handleSend = () => {
        addTaskLog({
            "description": sendValue,
            "taskId": params.get('id'),
            "enableFlag": '1'
        }).then(res => {
            // 加入当前集合中
            setTaskLogList([...taskLogList, res]);
        })
        setSendValue('')
        textAreaRef.current.focus();
    };

    async function fetchMessages() {
        // 获取之前的日志信息，根据日期分组排序，遍历map
        const res = await listTaskLog(JSON.stringify({
            "pageSize": 20,
            "sortList": [{"direction": "DESC", "property": "createdDate"}],
            "data": {
                "andList": [{
                    "name": "taskId",
                    "operateType": "=",
                    "value": params.get('id')
                }, {
                    "name": "createdDate",
                    "operateType": "<",
                    "value": pageTime
                }]
            }
        }))
        console.log({res})
        if (res.content.length > 0) {
            setPageTime(res.content[res.content.length - 1].createdDate)
            setTaskLogList([...taskLogList, ...res.content]);
        } else if (res.page.totalElements === 0) {
            setHasMore(false);
        }
    }

    useEffect(() => {
        setPageTime(new Date())
        textAreaRef.current.focus();
        fetchMessages()
    }, [])

    function NoNeed() {
        return <s>失效</s>
    }

    return <Fragment>
        <div className="log-task-detail">
            <Tabs defaultActiveKey='need' onChange={(key) => setCurrentShow(key)}>
                <Tabs.Tab title='全部' key='all'/>
                <Tabs.Tab title='有效' key='need'/>
                <Tabs.Tab key='noNeed' title=<NoNeed/> />
            </Tabs>
            <div id="scrollableDiv" className="log-detail"
            >
                <InfiniteScroll
                    dataLength={taskLogList.length}
                    next={fetchMessages}
                    hasMore={hasMore}
                    inverse={true} // 设置为 true 来反转列表的方向，使新消息在底部
                    loader={<h4>加载中...</h4>}
                    style={{display: 'flex', flexDirection: 'column-reverse'}}
                    scrollableTarget="scrollableDiv"
                    endMessage={
                        <Divider>没有更久远的信息了</Divider>
                    }
                >
                    {
                        Array.from(taskLogMapMemory.keys()).map(key => {
                            return (<Fragment>
                                {taskLogMapMemory.get(key).map(taskLog => {
                                    return <div key={taskLog.id}
                                                style={{display: 'flex', width: "80%", padding: "2px 20px"}}>
                                    <span className="detail-line" onClick={() => {
                                        if (taskLog.enableFlag === '0') {
                                            actions[1] = {text: '有效', key: 'need'}
                                        } else {
                                            actions[1] = {text: '失效', key: 'noNeed'}
                                        }
                                        // setActions([...actions])
                                        setCurrentTask(taskLog)
                                        setActionSheetVisible(true)
                                    }}>
                                        {taskLog.description}</span>
                                    </div>
                                })}
                                {dayjs().isSame(dayjs(key), "date") ?
                                    <div key={key} style={{width: "100%", padding: "0 40px"}}>
                                        <Divider>今日</Divider>
                                    </div> :
                                    <div key={key} style={{width: "100%", padding: "0 20px"}}>
                                        <Divider>{key}</Divider>
                                    </div>}
                            </Fragment>)
                        })
                    }
                </InfiniteScroll>
            </div>
            <div className="log-send">
                <TextArea
                    ref={(node) => {
                        textAreaRef.current = node;
                    }}
                    value={sendValue}
                    onChange={(val) => setSendValue(val)}
                    placeholder='输入日记心得，长按日记心得有惊喜'
                    rows={1}
                    autoSize={{minRows: 1, maxRows: 5}}
                    maxLength={100}
                    onKeyDown={event => {
                        if (event.key === 'Enter') {
                            handleSend();
                            // 阻止换行符插入
                            event.preventDefault();
                        }
                    }}
                />
                <Button
                    className="send-button"
                    onClick={handleSend}
                >发送
                </Button>
                <ActionSheet
                    extra='请选择你要进行的操作'
                    cancelText='取消'
                    visible={actionSheetVisible}
                    actions={actions}
                    onClose={() => setActionSheetVisible(false)}
                    onAction={action => {
                        if (action.key === 'copy') {
                            copyTextToClipboard(currentTask.description)
                        } else if (action.key === 'delete') {
                            Dialog.confirm({
                                content: '确定要删除吗？', onConfirm: () => {
                                    deleteTaskLogById(currentTask.id).then(result => {
                                        setTaskLogList(taskLogList.filter(log => log.id !== currentTask.id))
                                    })
                                }
                            })
                        } else if (action.key === 'noNeed') {
                            updateTaskLogEnableById('0', currentTask.id).then(result => {
                                setTaskLogList(taskLogList.map(log => {
                                        if (log.id === currentTask.id) {
                                            log.enableFlag = '0'
                                        }
                                        return log;
                                    })
                                )
                            });
                        } else if (action.key === 'need') {
                            updateTaskLogEnableById('1', currentTask.id).then(result => {
                                setTaskLogList(taskLogList.map(log => {
                                        if (log.id === currentTask.id) {
                                            log.enableFlag = '1'
                                        }
                                        return log;
                                    })
                                )
                            });
                        } else if (action.key === 'addTask') {
                            // 跳转到新增和新建下一条差不多，设置pidArray
                            // 获取父任务信息
                            Dialog.confirm({
                                content: '确定要创建任务吗？', onConfirm: () => {
                                    getPTask(params.get('id')).then(res => {
                                        let parentMessageVOList = res[0].parentMessageVOList;
                                        console.log({res, parentMessageVOList});
                                        // 进入新增页面
                                        navigate(`/detail/addTask?
                                        pName=${parentMessageVOList[parentMessageVOList.length - 1].name}&pidArray=${parentMessageVOList.map(parent => parent.id).join(',')}&description=${currentTask.description}`)
                                    })
                                }
                            })
                        }
                        setActionSheetVisible(false)
                    }}
                />
            </div>
        </div>

    </Fragment>

}