export function copyTextToClipboard(text) {
    return new Promise((resolve, reject) => {
        if (navigator.clipboard && window.isSecureContext) {
            // 尝试使用现代剪贴板 API
            navigator.clipboard.writeText(text)
                .then(() => resolve('文本已成功复制到剪贴板'))
                .catch((err) => reject(`无法复制文本：${err.message}`));
        } else {
            // 回退到旧方法
            fallbackCopyTextToClipboard(text)
                .then(message => resolve(message))
                .catch(err => reject(err));
        }
    });
}

function fallbackCopyTextToClipboard(text) {
    return new Promise((resolve, reject) => {
        const textArea = document.createElement("textarea");
        textArea.value = text;

        // 避免 CSS 样式影响滚动条
        textArea.style.position = 'fixed';
        textArea.style.left = '-9999px';
        document.body.appendChild(textArea);
        textArea.focus();
        textArea.select();

        try {
            const successful = document.execCommand('copy');
            const msg = successful ? '文本已成功复制到剪贴板' : '无法复制文本';
            resolve(msg);
        } catch (err) {
            reject(`无法复制文本：${err.message}`);
        }

        document.body.removeChild(textArea);
    });
}